// ignore_for_file: prefer_function_declarations_over_variables

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_common/utils/src/image_utils.dart';

debounce(Function fn, [int t = 500]) {
  Timer? _debounce;
  return () {
    if (_debounce?.isActive ?? false) {
      _debounce?.cancel();
    }

    _debounce = Timer(Duration(milliseconds: t), () {
      fn();
    });
  };
}

throttle(
  Future Function() func,
) {
  bool enable = true;
  Function target = () {
    if (enable == true) {
      enable = false;
      func().then((_) {
        enable = true;
      });
    }
  };
  return target;
}

class PrimaryButton extends StatelessWidget {
  const PrimaryButton({Key? key, required this.onPress, required this.title, this.disable = false, this.backgroundColor = const Color(0xff2B6BD8), this.textColor = Colors.white}) : super(key: key);

  final Function() onPress;
  final String title;
  final bool disable;
  final Color backgroundColor;
  final Color textColor;

  @override
  Widget build(BuildContext context) {
    return ElevatedButton(
      onPressed: !disable
          ? throttle(() async {
              onPress();
              await Future.delayed(const Duration(milliseconds: 500));
            })
          : null,
      style: ElevatedButton.styleFrom(
        shadowColor: Colors.transparent,
        primary: disable ? const Color.fromARGB(128, 43, 107, 216) : backgroundColor,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(4.r),
        ),
      ),
      child: SizedBox(
        height: 44.h,
        child: Center(
          child: Text(
            title,
            style: TextStyle(color: textColor, fontSize: 18.sp),
          ),
        ),
      ),
    );
  }
}

class OutBorderButton extends StatelessWidget {
  const OutBorderButton(
      {Key? key,
      required this.onPress,
      required this.title,
      this.disable = false,
      this.height = 44,
      this.minWidth = 88,
      this.titleStyle,
      this.decoration,
      this.disableColor = const Color(0xff969799),
      this.borderColor = const Color(0xff1989FA)})
      : super(key: key);

  final Function() onPress;
  final String title;
  final Color borderColor;
  final Color disableColor;
  final bool disable;
  final double height;
  final double minWidth;
  final TextStyle? titleStyle;
  final BoxDecoration? decoration;

  @override
  Widget build(BuildContext context) {
    final TextStyle style = titleStyle ?? TextStyle(color: disable ? disableColor : borderColor, fontSize: 18.sp);
    return OutlinedButton(
      onPressed: !disable
          ? debounce(() {
              onPress();
            })
          : null,
      style: OutlinedButton.styleFrom(
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(4.r),
          ),
          side: BorderSide(width: 1, color: disable ? disableColor : borderColor),
          padding: const EdgeInsets.all(0)),
      child: Container(
        height: height.h,
        constraints: BoxConstraints(minWidth: minWidth.w),
        decoration: decoration,
        child: Center(
          child: Text(
            title,
            style: style,
          ),
        ),
      ),
    );
  }
}

class AddListButton extends StatelessWidget {
  const AddListButton({
    Key? key,
    required this.onPress,
    required this.title,
    this.borderSideColor,
    this.backgroundColor,
  }) : super(key: key);
  final Function() onPress;
  final String title;
  final Color? borderSideColor;
  final Color? backgroundColor;

  @override
  Widget build(BuildContext context) {
    return TextButton.icon(
      onPressed: onPress,
      icon: const Icon(
        Icons.add,
        size: 24,
        color: Color(0xff1989FA),
      ),
      label: Text(
        title,
        style: TextStyle(fontSize: 15.sp, color: const Color(0xff1989FA)),
      ),
      style: ButtonStyle(
          shape: MaterialStateProperty.all(RoundedRectangleBorder(side: BorderSide(color: borderSideColor ?? const Color(0xff1989FA)), borderRadius: BorderRadius.circular(4.r))),
          backgroundColor: MaterialStateProperty.resolveWith((states) {
            if (states.contains(MaterialState.pressed)) {
              return Colors.blue;
            }
            return backgroundColor ?? const Color(0xffE6F2FE);
          }),
          minimumSize: MaterialStateProperty.all(const Size(double.infinity, 36))),
    );
  }
}

class GridButton extends StatelessWidget {
  const GridButton({Key? key, required this.title, required this.onTap, required this.icon}) : super(key: key);
  final String title;
  final IconData icon;
  final Function() onTap;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Ink(
        //padding: EdgeInsets.all(6.r),
        child: Column(
          children: [
            Stack(
              alignment: AlignmentDirectional.center,
              children: [
                Image(
                  image: ImageUtils.getAssetImage("assets/images/icon_bg.png", package: "tengits_common"),
                  width: 40,
                  height: 40,
                  fit: BoxFit.contain,
                ),
                Icon(
                  icon,
                  color: const Color(0xff1989FA),
                  size: 24,
                ),
              ],
            ),
            const SizedBox(
              height: 6,
            ),
            Text(
              title,
              maxLines: 1,
              style: TextStyle(color: const Color(0xff323233), fontSize: 12.sp),
            ),
          ],
        ),
      ),
    );
  }
}

class ImageButton extends StatelessWidget {
  const ImageButton({Key? key, required this.title, required this.onTap, required this.image}) : super(key: key);
  final String title;
  final Widget image;
  final Function() onTap;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Ink(
        child: Container(
          padding: EdgeInsets.symmetric(horizontal: 19.w, vertical: 16.h),
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(8),
          ),
          child: Column(
            children: [
              image,
              const SizedBox(
                height: 8,
              ),
              Text(
                title,
                style: TextStyle(color: const Color(0xff323233), fontSize: 16.sp),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

class CenterDockerButton extends StatelessWidget {
  const CenterDockerButton({Key? key, required this.onPressed}) : super(key: key);

  final Function() onPressed;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onPressed,
      child: Ink(
        child: Container(
          width: 54,
          height: 54,
          decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.circular(27.0),
              //border: Border.all(color: const Color(0x4CCACACA)),
              boxShadow: const [BoxShadow(color: Color(0x4CCACACA), offset: Offset(0, -4), blurRadius: 5.0)]),
          child: Center(
            child: Container(
              width: 46,
              height: 46,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(23), gradient: const LinearGradient(colors: [Color(0xFF5DACFB), Color(0xFF2B6BD8)], begin: Alignment.topCenter, end: Alignment.bottomCenter)),
              child: const Icon(
                Icons.add,
                size: 14,
                color: Colors.white,
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class FloatingAddButton extends StatelessWidget {
  const FloatingAddButton({
    Key? key,
    required this.onPressed,
    this.widget,
    this.decoration,
  }) : super(key: key);

  final Function() onPressed;
  final Widget? widget;
  final Decoration? decoration;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onPressed,
      child: Ink(
        child: SizedBox(
          width: 56.r,
          height: 56.r,
          child: Center(
            child: Container(
              width: 46.r,
              height: 46.r,
              decoration: decoration ??
                  BoxDecoration(
                    borderRadius: BorderRadius.circular(23.r),
                    gradient: const LinearGradient(
                      colors: [Color(0xFF5DACFB), Color(0xFF2B6BD8)],
                      begin: Alignment.topCenter,
                      end: Alignment.bottomCenter,
                    ),
                  ),
              child: widget ??
                  Icon(
                    Icons.add,
                    size: 24.r,
                    color: Colors.white,
                  ),
            ),
          ),
        ),
      ),
    );
  }
}

class TextIconButton extends StatelessWidget {
  const TextIconButton({
    Key? key,
    required this.title,
    required this.icon,
    required this.onTap,
    this.ltr = true,
    this.constraints,
    this.decoration,
    this.space,
  }) : super(key: key);
  final Text title;
  final Widget icon;
  final void Function() onTap;
  final BoxConstraints? constraints;
  final BoxDecoration? decoration;
  final bool ltr;
  final double? space;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Ink(
        child: Container(
          constraints: constraints,
          decoration: decoration,
          child: Row(
            textDirection: ltr ? TextDirection.ltr : TextDirection.rtl,
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              title,
              SizedBox(
                width: space?.w ?? 3.w,
              ),
              icon
            ],
          ),
        ),
      ),
    );
  }
}
