import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tengits_common/utils/src/image_utils.dart';

import 'textFlag.dart';

class WorkItemCard extends StatelessWidget {
  const WorkItemCard({
    Key? key,
    required this.title,
    this.count,
    required this.assetImage,
    required this.onTap,
    this.width,
    this.height,
    this.startColor = Colors.white,
    this.endColor = const Color(0xFFF6FDFF),
    this.borderColor = Colors.white,
    this.fontFamily = "dinFont"
  }) : super(key: key);

  final String title;
  final dynamic count;
  final String assetImage;
  final double? width;
  final double? height;
  final Function() onTap;
  final Color startColor;
  final Color endColor;
  final Color borderColor;
  final String fontFamily;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Ink(
        child: Stack(
          alignment: Alignment.bottomRight,
          children: [
            Container(
              width: width ?? 166.w,
              height: height,
              padding: EdgeInsets.symmetric(horizontal: 16.w, vertical: 16.h),
              decoration: BoxDecoration(
                gradient: LinearGradient(
                  colors: [startColor, endColor],
                  begin: Alignment.topLeft,
                  end: Alignment.bottomRight,
                ),
                border: Border.all(color: borderColor),
                borderRadius: BorderRadius.circular(8.r),
              ),
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text(title, style: TextStyle(color: const Color(0xff323233), fontSize: 16.sp)),
                  SizedBox(height: 8.h),
                  if(count != null)
                    if((count is String && count.isNotEmpty))
                      Text(
                        "$count",
                        style: TextStyle(
                          fontFamily: fontFamily,
                          color: const Color(0xFF323233),
                          fontSize: 36.sp,
                          fontWeight: FontWeight.bold,
                        ),
                      )
                    else if((count is Widget))
                     (count as Widget)
                ],
              ),
            ),
            Positioned(
              child: Image(
                image: ImageUtils.getAssetImage(assetImage),
                fit: BoxFit.fill,
                width: 50.r,
                height: 50.r,
              ),
              right: 4.w,
              bottom: 12.h,
            )
          ],
        ),
      ),
    );
  }
}

class WorkSpaceCard extends StatelessWidget {
  const WorkSpaceCard({Key? key, required this.title, required this.children, 
                        this.crossCount =4, this.mainSpacing=16, this.crossSpacing=16, this.ratio=0.88, 
                        this.padding=const EdgeInsets.all(16)}) : super(key: key);

  final String title;
  final List<Widget> children;
  final int crossCount;
  final double mainSpacing;
  final double crossSpacing;
  final double ratio;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.only(top: 12.h),
      decoration: BoxDecoration(color: Colors.white, borderRadius: BorderRadius.circular(8.r)),
      child: Column(
        children: [
          TextFlag(
            title: title,
          ),
          SeparationLine(
            margin: EdgeInsets.only(top: 12.h),
          ),
          GridView.count(
            physics: const NeverScrollableScrollPhysics(),
            crossAxisCount: crossCount,
            mainAxisSpacing: mainSpacing.w,
            crossAxisSpacing: crossSpacing.h,
            childAspectRatio: ratio,
            children: children,
            shrinkWrap: true,
            padding: padding,
          ),
        ],
      ),
    );
  }
}

class OperatorCard<T> extends StatelessWidget {
  const OperatorCard({
    Key? key,
    required this.title,
    this.isRequired = false,
    this.isSepateLine = true,
    this.trailing,
    this.margin,
    this.containPadding,
    this.separateMargin,
    this.contentPadding,
    this.decoration,
    this.child,
    this.alignment,
  }) : super(key: key);

  final T title;
  final bool isRequired;
  final Widget? trailing;
  final EdgeInsetsGeometry? margin;
  final EdgeInsetsGeometry? containPadding;
  final EdgeInsetsGeometry? separateMargin;
  final EdgeInsetsGeometry? contentPadding;
  final Widget? child;
  final BoxDecoration? decoration;
  final bool isSepateLine;
  final AlignmentGeometry? alignment;

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 0,
      color: Colors.white,
      margin: margin ?? EdgeInsets.only(bottom: 16.h),
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(8.r)),
      child: Container(
        padding: containPadding ?? EdgeInsets.symmetric(vertical: 12.h),
        child: Column(
          children: [
            TextFlag(
              title: title,
              isRequired: isRequired,
              trailing: trailing,
            ),
            Offstage(
              offstage: !isSepateLine,
              child: SeparationLine(
                margin: separateMargin ?? EdgeInsets.only(top: 12.h, bottom: 8.h),
              ),
            ),
            Offstage(
              offstage: child == null,
              child: Container(
                padding: contentPadding ?? EdgeInsets.symmetric(horizontal: 16.w),
                decoration: decoration,
                alignment: alignment,
                child: child,
              ),
            )
          ],
        ),
      ),
    );
  }
}

class ButtonOperatorCard extends StatelessWidget {
  const ButtonOperatorCard({Key? key, required this.buttons, this.isSpaceAround = false}) : super(key: key);

  final List<Widget> buttons;
  final bool isSpaceAround;

  @override
  Widget build(BuildContext context) {
    List<Widget> renders = [];
    for (int i = 0; i < buttons.length; i++) {
      //均等排列
      if (isSpaceAround == true) {
        renders.add(Expanded(child: buttons[i]));
      } else {
        if (i == buttons.length - 1) {
          renders.add(Expanded(child: buttons[i]));
        } else {
          renders.add(buttons[i]);
        }
      }
      // 按钮间隙
      if (i < buttons.length - 1) {
        renders.add(SizedBox(
          width: 8.w,
        ));
      }
    }

    return Offstage(
      offstage: renders.isEmpty,
      child: Container(
        width: double.infinity,
        padding: EdgeInsets.only(left: 16.w, right: 16.w, top: 12.h, bottom: (12.h + MediaQuery.of(context).padding.bottom)),
        decoration: const BoxDecoration(color: Colors.white, border: Border(top: BorderSide(color: Color(0xffF2F3F5)))),
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: renders,
        ),
      ),
    );
  }
}

class CardHeader<T> extends StatelessWidget {
  const CardHeader({
    Key? key,
    required this.title,
    this.trailing,
  }) : super(key: key);
  final T title;
  final Widget? trailing;
  @override
  Widget build(BuildContext context) {
    return ClipRRect(
      borderRadius: BorderRadius.only(topLeft: Radius.circular(8.r), topRight: Radius.circular(8.r)),
      child: Container(
        padding: EdgeInsets.symmetric(vertical: 12.h),
        decoration: const BoxDecoration(color: Colors.white, border: Border(bottom: BorderSide(color: Color(0xffF7F8FA)))),
        child: TextFlag(
          title: title,
          trailing: trailing,
        ),
      ),
    );
  }
}
