// ignore_for_file: file_names, unnecessary_string_interpolations, avoid_print, unnecessary_brace_in_string_interps

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';

class TLTextField extends StatefulWidget {
  const TLTextField({
    Key? key,
    this.defaultValue,
    this.hintText,
    this.maxLength,
    this.style,
    this.hintStyle,
    this.textAlign,
    this.keyboardType,
    this.inputFormatters,
    this.onChange,
    this.enable = true,
    this.onSubmit,
    this.focusNode,
    this.contentPadding,
    this.border,
    this.enableBorder,
    this.disableBorder,
    this.focusBorder,
    this.onDispose,
    this.onFocusChange,
    this.inputDecoration,
  }) : super(key: key);

  final String? defaultValue;
  final String? hintText;
  final int? maxLength; //[maxLength]限定文本长度，为null时不限
  final TextStyle? style;
  final TextStyle? hintStyle;
  final TextAlign? textAlign;
  final TextInputType? keyboardType;
  final List<TextInputFormatter>? inputFormatters;
  final Function(String)? onChange;
  final bool enable;
  final Function(String)? onSubmit;
  final FocusNode? focusNode;
  final EdgeInsetsGeometry? contentPadding;
  final InputBorder? border;
  final InputBorder? enableBorder;
  final InputBorder? disableBorder;
  final InputBorder? focusBorder;
  final Function()? onDispose;
  final Function(bool)? onFocusChange;
  final InputDecoration? inputDecoration;

  @override
  State<TLTextField> createState() => _TLTextFieldState();
}

class _TLTextFieldState extends State<TLTextField> {
  //
  late TextEditingController textEditingController;

  @override
  void initState() {
    super.initState();

    textEditingController = TextEditingController(text: widget.defaultValue ?? "")
      ..addListener(() {
        widget.onChange?.call(textEditingController.text);
      });
  }

  @override
  void didUpdateWidget(TLTextField oldWidget) {
    if (oldWidget.defaultValue != widget.defaultValue) {
      TextSelection position = textEditingController.selection.copyWith();
      textEditingController = TextEditingController(text: widget.defaultValue ?? "");
      //invalid text selection: TextSelection.collapsed(offset: ??s, affinity: TextAffinity.downstream, isDirectional: false) fmq
      if (textEditingController.isSelectionWithinTextBounds(position)) {
        textEditingController.selection = position;
      }
      textEditingController.addListener(() {
        widget.onChange?.call(textEditingController.text);
      });
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    textEditingController.dispose();
    widget.onDispose?.call();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Focus(
      onFocusChange: ((value) {
        widget.onFocusChange?.call(value);
      }),
      child: TextField(
        focusNode: widget.focusNode,
        textAlign: widget.textAlign ?? TextAlign.start,
        controller: textEditingController,
        style: widget.style ?? TextStyle(color: const Color(0xff323233), fontSize: 16.sp),
        enabled: widget.enable,
        decoration: widget.inputDecoration ?? InputDecoration(
          hintMaxLines: 2,
          hintText: widget.hintText ?? "请输入",
          hintStyle: widget.hintStyle ?? TextStyle(color: const Color(0xff969799), fontSize: 14.sp),
          border: widget.border ?? InputBorder.none,
          enabledBorder: widget.enableBorder ?? InputBorder.none,
          disabledBorder: widget.disableBorder ?? InputBorder.none,
          focusedBorder: widget.focusBorder ?? InputBorder.none,
          contentPadding: widget.contentPadding ?? EdgeInsets.zero,
          isDense: true,
        ),
        minLines: 1,
        maxLines: 4,
        keyboardType: widget.keyboardType,
        inputFormatters: (widget.inputFormatters ?? [])..addAll([LengthLimitingTextInputFormatter(widget.maxLength)]),
        autofocus: false,
        onSubmitted: (text) {
          widget.onSubmit?.call(text);
        },
      ),
    );
  }
}

///忽略特殊字符
class TLIgnoreOtherFormatter extends TextInputFormatter {
  final _regExp = r"^[\u4E00-\u9FA5A-Za-z0-9_]+$";

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///忽略表情
class TLBlacklistingTextInputFormatter extends TextInputFormatter {
  final _regExp = r"^[^\\u0020-\\u007E\\u00A0-\\u00BE\\u2E80-\\uA4CF\\uF900-\\uFAFF\\uFE30-\\uFE4F\\uFF00-\\uFFEF\\u0080-\\u009F\\u2000-\\u201f\r\n]+$";

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///只能输入数字和小写字母
class TLOnlyInputNumberAndLowWorkFormatter extends TextInputFormatter {
  final _regExp = r"^[a-z0-9_]+$";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///只能输入数字和字母
class TLOnlyInputNumberAndWorkFormatter extends TextInputFormatter {
  final _regExp = r"^[A-Za-z0-9_]+$";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

///只能输入字母
class TLOnlyInputWorkFormatter extends TextInputFormatter {
  final _regExp = r"^[A-Za-z]+$";
  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    if (newValue.text.isNotEmpty) {
      if (RegExp(_regExp).firstMatch(newValue.text) != null) {
        return newValue;
      }
      return oldValue;
    }
    return newValue;
  }
}

class TLNumberTextField extends TLTextField {
  TLNumberTextField({
    Key? key,
    String? defaultValue,
    String? hintText,
    int? maxLength,
    int? maxIntegerLength,
    int? maxDecimalLength,
    TextStyle? style,
    TextStyle? hintStyle,
    TextAlign? textAlign,
    TextInputType? keyboardType = const TextInputType.numberWithOptions(decimal: true),
    List<TextInputFormatter>? inputFormatters,
    Function(String)? onChange,
    bool enable = true,
    Function(String)? onSubmit,
    FocusNode? focusNode,
    EdgeInsetsGeometry? contentPadding,
    InputBorder? border,
    InputBorder? enableBorder,
    InputBorder? disableBorder,
    InputBorder? focusBorder,
    final Function()? onDispose,
    final Function(bool)? onFocusChange,
  }) : super(
          key: key,
          defaultValue: defaultValue,
          hintText: hintText,
          maxLength: maxLength,
          style: style,
          hintStyle: hintStyle,
          textAlign: textAlign,
          keyboardType: keyboardType,
          inputFormatters: inputFormatters ??
              [
                //文本长度限制
                LengthLimitingTextInputFormatter(maxLength),
                //只允许输入小数
                FilteringTextInputFormatter.allow(RegExp("[0-9.]")),
                //FilteringTextInputFormatter.allow(RegExp("[0-9.]")),但这种限制会有问题，比如可以输入0.1.1这种其实就不是小数了，并且不能限制小数点的输入位数。
                TLNumberTextInputFormatter(maxIntegerLength: maxIntegerLength, maxDecimalLength: maxDecimalLength, isAllowDecimal: true),
              ],
          onChange: onChange,
          enable: enable,
          onSubmit: onSubmit,
          focusNode: focusNode,
          contentPadding: contentPadding,
          border: border,
          enableBorder: enableBorder,
          disableBorder: disableBorder,
          focusBorder: focusBorder,
          onDispose: onDispose,
          onFocusChange: onFocusChange,
        );
}

class TLNumberTextInputFormatter extends TextInputFormatter {
  /// [maxIntegerLength]限定整数的最大位数，为null时不限
  /// [maxDecimalLength]限定小数点的最大位数，为null时不限
  /// [isAllowDecimal]是否可以为小数，默认是可以为小数，也就是可以输入小数点
  final int? _maxIntegerLength;
  final int? _maxDecimalLength;
  final bool _isAllowDecimal;

  TLNumberTextInputFormatter({int? maxIntegerLength, int? maxDecimalLength, bool isAllowDecimal = true})
      : _maxIntegerLength = maxIntegerLength,
        _maxDecimalLength = maxDecimalLength,
        _isAllowDecimal = isAllowDecimal;

  @override
  TextEditingValue formatEditUpdate(TextEditingValue oldValue, TextEditingValue newValue) {
    String? value = newValue.text.trim();

    ///去掉前后空格
    int selectionIndex = newValue.selection.end;
    if (_isAllowDecimal) {
      if (value == '.') {
        value = '0.';
        selectionIndex++;
      } else if (value != '' && _isToDoubleError(value)) {
        ///不是double输入数据
        return _oldTextEditingValue(oldValue);
      }

      ///包含小数点
      if (value.contains('.')) {
        int pointIndex = value.indexOf('.');
        String beforePoint = value.substring(0, pointIndex);
        print('$beforePoint');
        String afterPoint = value.substring(pointIndex + 1, value.length);
        print('$afterPoint');

        ///小数点前面没内容补0
        if (beforePoint.isEmpty) {
          value = '0.${afterPoint}';
          selectionIndex++;
        } else {
          ///限定整数位数
          if (null != _maxIntegerLength) {
            if (beforePoint.length > _maxIntegerLength!) {
              return _oldTextEditingValue(oldValue);
            }
          }
        }

        ///限定小数点位数
        if (null != _maxDecimalLength) {
          if (afterPoint.length > _maxDecimalLength!) {
            return _oldTextEditingValue(oldValue);
          }
        }
      } else {
        ///限定整数位数
        if (null != _maxIntegerLength) {
          if (value.length > _maxIntegerLength!) {
            return _oldTextEditingValue(oldValue);
          }
        }
      }
    } else {
      if (value.contains('.') || (value != '' && _isToDoubleError(value)) || (null != _maxIntegerLength && value.length > _maxIntegerLength!)) {
        return _oldTextEditingValue(oldValue);
      }
    }

    return TextEditingValue(
      text: value,
      selection: TextSelection.collapsed(offset: selectionIndex),
    );
  }

  ///返回旧的输入内容
  TextEditingValue _oldTextEditingValue(TextEditingValue oldValue) {
    return TextEditingValue(
      text: oldValue.text,
      selection: TextSelection.collapsed(offset: oldValue.selection.end),
    );
  }

  ///输入内容不能解析成double
  bool _isToDoubleError(String value) {
    try {
      double.parse(value);
    } catch (e) {
      return true;
    }
    return false;
  }
}
